"""
Extra mods required: pyyaml, requests, jsonschema
"""

import os
import signal
import sys
import argparse
import requests
import time
from pathlib import Path
from .config import choose_file, load_yaml
import jsonschema

PROPERTY_MAP_SCHEMA = {
    'type': 'object',
    'properties': {
        'path': {
            'type': 'string'
        },
        'listen': {
            'type': 'string'
        },
        'pid-file': {
            'type': 'string'
        },
        'start-failed-after': {
            'type': 'integer',
            'minimum': 0.1
        },
        'force-stop-after': {
            'type': 'integer',
            'minimum': 0.1
        },
        'launch-debug': {
            'type': 'boolean',
        },
        'extra-options': {
            'type': 'string'
        }
    },
    'additionalProperties': False
}


def manage_gunicorn_app(app,
                        app_dir='.',
                        name=None,
                        version=None,
                        build=None,
                        default_port=8081,
                        app_class=None,
                        api_uri='/',
                        health_check_uri='/ping'):
    """
    Manage gunicorn-based apps

    Args:
        app: app code
        app_dir: app directory
        name: app name
        default_port: app default listen port
        app_class: launch class for app
        api_uri: app API uri
    """

    def printfl(*args, **kwargs):
        print(*args, **kwargs)
        sys.stdout.flush()

    os.chdir(app_dir)
    if name is None:
        name = app.capitalize()
    if app_class is None:
        app_class = f'{app}.server:app'
    ap = argparse.ArgumentParser()
    cmds = ['start', 'stop', 'restart', 'status', 'launch']
    if version:
        cmds.append('version')
    ap.add_argument('command', choices=cmds)
    ap.add_argument('--config-file',
                    metavar='FILE',
                    help='alternative config file')
    ap.add_argument('-D',
                    '--debug',
                    help='Force debug mode',
                    action='store_true')
    a = ap.parse_args()

    if a.command == 'version':
        print('{}{}{}'.format(name, f' {version}' if version else '',
                              f' {build}' if build else ''))
        sys.exit(0)

    app_env_name = f'{app.upper()}_CONFIG'

    config_file = choose_file(a.config_file,
                              env=app_env_name,
                              choices=[
                                  f'{app_dir}/etc/{app}.yml',
                                  f'/opt/{app}/etc/{app}.yml',
                                  f'/usr/local/etc/{app}.yml'
                              ])
    config = load_yaml(config_file)[app].get('gunicorn', {})
    jsonschema.validate(config, schema=PROPERTY_MAP_SCHEMA)

    pidfile = config.get('pid-file', f'/tmp/{app}.pid')
    api_listen = config.get('listen', f'0.0.0.0:{default_port}')
    api_url = api_listen.replace('0.0.0.0', '127.0.0.1')
    start_failed_after = config.get('start-failed-after', 10)
    force_stop_after = config.get('force-stop-after', 10)
    debug_mode = a.debug
    launch_debug = True if debug_mode else config.get('launch-debug')

    def get_app_pid():
        if Path(pidfile).exists():
            with open(pidfile) as fh:
                return int(fh.read())
        else:
            return None

    def get_app_status(pid):
        try:
            os.kill(pid, 0)
            return True
        except:
            return False

    def health_check():
        try:
            r = requests.get(f'http://{api_url}{health_check_uri}')
            if not r.ok:
                raise RuntimeError
            return True
        except:
            return False

    def status():
        pid = get_app_pid()
        if pid and get_app_status(pid):
            if health_check():
                print(f'{name} is running. API: http://{api_listen}{api_uri}')
            else:
                print(f'{name} is dead')
            return False
        print(f'{name} is not running')
        return False

    def stop_server():
        pid = get_app_pid()
        if pid and get_app_status(pid):
            printfl(f'Stopping {name}...', end='')
            os.kill(pid, signal.SIGTERM)
            c = 0
            while Path(pidfile).exists():
                time.sleep(1)
                printfl('.', end='')
                c += 1
                if c > force_stop_after:
                    os.kill(pid, signal.SIGKILL)
                    print('KILLED')
                    return True
            print('stopped')
            return True
        else:
            return False

    def start_server(launch=False):
        pid = get_app_pid()
        if pid and get_app_status(pid):
            print(f'{name} is already running')
        else:
            if not launch: printfl(f'Starting {name}...', end='')
            xopts = config.get('extra-options', '')
            if xopts and launch:
                import re
                xopts = re.sub(r'--log-file .* ', '', xopts + ' ') + ' '
                xopts = f' {xopts} '.replace(' --log-syslog ', ' ')
            code = os.system(
                ('{gunicorn} {daemon} -e {config_env} --pid {pidfile}'
                 ' -b {api_listen} {xopts} {debug} {app_class}').format(
                     gunicorn=os.getenv('GUNICORN',
                                        config.get('path', 'gunicorn')),
                     daemon='' if launch else '-D',
                     config_env=f'{app_env_name}={config_file}',
                     pidfile=pidfile,
                     api_listen=api_listen,
                     xopts=xopts,
                     debug='--log-level DEBUG' if
                     (launch and launch_debug) or debug_mode else '',
                     app_class=app_class))
            if code:
                if not launch: print(f'FAILED ({code})')
                return False
            elif not launch:
                c = 0
                while not Path(pidfile).exists() or not health_check():
                    c += 1
                    time.sleep(1)
                    printfl('.', end='')
                    if c > start_failed_after:
                        print('FAILED')
                        return False
                print('started')
                return True
            else:
                return True

    def restart_server():
        if stop_server():
            time.sleep(1)
        return start_server()

    if a.command == 'start':
        sys.exit(0 if start_server() else 1)
    if a.command == 'launch':
        sys.exit(0 if start_server(launch=True) else 1)
    elif a.command == 'stop':
        stop_server()
        sys.exit(0)
    elif a.command == 'restart':
        sys.exit(0 if restart_server() else 1)
    elif a.command == 'status':
        sys.exit(0 if status() else 1)



"""
Extra mods required: sqlalchemy, msgpack (for KVStorage)
"""
import threading
import os
import datetime
from types import SimpleNamespace
from pyaltt2.crypto import gen_random_str
from pyaltt2.res import ResourceStorage
import pyaltt2.json as json
from functools import partial


def format_condition(f, kw=None, fields=None, cond=None):
    """
    Args:
        f: condition filter (dict name/value)
        kw: kwargs query dict
        fields: allowed fields
        cond: initial condition
    Returns:
        tuple cond (string), kw (dict). String can be safely used in SQL query,
        dict should be set as query kwargs
    """
    if kw is None:
        kw = {}
    else:
        kw = kw.copy()
    if fields is None:
        import re
        fmatch = re.compile('^[A-Za-z0-9\._-]*$')
    if cond is None:
        cond = ''
    for k, v in f.items():
        if (fields is None and
                not re.match(fmatch, k)) or (fields is not None and
                                             k not in fields):
            raise ValueError(k)
        eqs = ' is ' if v in (None, True, False) else '='
        if v is None:
            val = 'null'
        elif v is True:
            val = 'true'
        elif v is False:
            val = 'false'
        else:
            key = k.replace('.', '__').replace('-', '__')
            val = f':{key}'
            kw[key] = v
        if cond == '':
            cond = 'where '
        else:
            cond += ' and '
        cond += f'{k}{eqs}{val}'
    return cond, kw


class Database:
    """
    Database wrapper for SQLAlchemy
    """

    _clone_params = ('db', 'db_lock', 'g', 'rq_func')

    def __init__(self, dbconn=None, rq_func=None, **kwargs):
        """
        Args:
            dbconn: database connection string (for SQLite - only file name is
                required)
            rq_func: resource loader function (for query method)
            kwargs: additional engine options (ignored for SQLite)
        """
        if not dbconn:
            return
        import sqlalchemy as sa

        class _ForeignKeysListener(sa.interfaces.PoolListener):

            def connect(self, dbapi_con, con_record):
                try:
                    dbapi_con.execute('pragma foreign_keys=ON')
                except:
                    pass

        self.db_lock = threading.RLock()
        self.g = threading.local()
        self.rq_func = rq_func
        if dbconn.find('://') == -1:
            dbconn = 'sqlite:///' + os.path.expanduser(dbconn)
        if dbconn.startswith('sqlite:///'):
            self.db = sa.create_engine(dbconn,
                                       listeners=[_ForeignKeysListener()])
        else:
            self.db = sa.create_engine(dbconn, **kwargs)
        self._setup()

    def clone(self, **kwargs):
        """
        Clone database object

        Extra kwargs (db, db_lock, g, rq_func) are assigned to object as-is
        """
        o = Database()
        for c in self._clone_params:
            setattr(o, c, kwargs[c] if c in kwargs else getattr(self, c))
        o._setup()
        return o

    def _setup(self):
        self.use_lastrowid = self.db.name in ['sqlite', 'mysql']
        self.use_interval = self.db.name not in ['sqlite', 'mysql']
        self.parse_db_json = self.db.name in ['sqlite', 'mysql']
        self.name = self.db.name

    def __repr__(self):
        return self.db.__repr__()

    def __str__(self):
        return self.db.__str__()

    def _format_result(self, result, json_fields=[]):
        if result is not None:
            result = dict(result)
            if self.parse_db_json:
                for f in json_fields:
                    result[f] = json.loads(result[f])
        return result

    def _format_list(self, result, json_fields=[]):
        if self.parse_db_json and json_fields:
            for row in result:
                for f in json_fields:
                    row[f] = json.loads(row[f])
        return result

    def get_engine(self):
        """
        Get DB engine object
        """
        return self.db

    def list(self, *args, json_fields=[], **kwargs):
        """
        get self.execute result as list of dicts

        Args:
            json_fields: decode json fields if required
            other: passed as-is
        """
        return self._format_list(
            [dict(row) for row in self.execute(*args, **kwargs).fetchall()],
            json_fields=json_fields)

    def qlist(self, *args, json_fields=[], **kwargs):
        """
        get self.query result as list of dicts

        Args:
            json_fields: decode json fields if required
            other: passed as-is
        """
        return self._format_list(
            [dict(row) for row in self.query(*args, **kwargs).fetchall()],
            json_fields=json_fields)

    def connect(self):
        """
        Get thread-safe db connection
        """
        with self.db_lock:
            try:
                self.g.conn.execute('select 1')
                return self.g.conn
            except:
                self.g.conn = self.db.connect()
                return self.g.conn

    def execute(self, *args, _cr=False, **kwargs):
        """
        Execute SQL query

        Args:
            _cr: check result, raise LookupError if row count is zero
            other: passed to SQLAlchemy connection as-is
        """
        result = self.connect().execute(*args, **kwargs)
        if _cr and result.rowcount == 0:
            raise LookupError
        else:
            return result

    def query(self, q, qargs=[], qkwargs={}, _create=False, *args, **kwargs):
        """
        Execute SQL query by resource

        Args:
            q: resource name
            qargs, qkwargs: format query with args/kwargs
            other: passed as-is

        Requires rq_func
        """
        from sqlalchemy import text as sql
        q = self.rq_func(q)
        if qargs or qkwargs:
            q = q.format(*qargs, **qkwargs)
        if _create and not self.use_lastrowid:
            q += ' RETURNING id'
        return self.execute(sql(q), *args, **kwargs)

    def create(self, q, *args, **kwargs):
        """
        Execute (usually INSERT) query with self.execute and return row id

        row id must be in "id" field
        """
        if not self.use_lastrowid:
            q += ' RETURNING id'
        result = self.execute(q, *args, **kwargs)
        return result.lastrowid if self.use_lastrowid else result.fetchone().id

    def qcreate(self, q, *args, **kwargs):
        """
        Execute (usually INSERT) query with self.query and return row id

        row id must be in "id" field
        """
        result = self.query(q, _create=True, *args, **kwargs)
        return result.lastrowid if self.use_lastrowid else result.fetchone().id

    def lookup(self, *args, json_fields=[], **kwargs):
        """
        Get single db row, use self.execute

        Args:
            json_fields: decode json fields if required
            other: passed as-is

        Returns:
            single row as a dict
        Raises:
            LookupError: if nothing found
        """
        result = self._format_result(self.execute(*args, **kwargs).fetchone(),
                                     json_fields=json_fields)
        if result:
            return result
        else:
            raise LookupError

    def qlookup(self, *args, json_fields=[], **kwargs):
        """
        Get single db row, use self.query

        Returns:
            single row as a dict
        Raises:
            LookupError: if nothing found
        """
        result = self._format_result(self.query(*args, **kwargs).fetchone(),
                                     json_fields=json_fields)
        if result:
            return result
        else:
            raise LookupError


class KVStorage:
    """
    Simple key-value database storage
    """

    def __init__(self, db, table_name='kv'):
        """
        Args:
            db: pyaltt2.db.Database
            table_name: storage table name (default: kv)
        """
        from sqlalchemy import (MetaData, Table, VARCHAR, DateTime, LargeBinary,
                                Column)
        if 'mysql' in db.get_engine().name:
            from sqlalchemy.dialects.mysql import DATETIME, LONGBLOB
            DateTime = partial(DATETIME, fsp=6)
            LargeBinary = LONGBLOB
        meta = MetaData()
        self.db = db.clone()
        rs = ResourceStorage(mod='pyaltt2')
        self.db.rq_func = partial(rs.get, resource_subdir='sql', ext='sql')
        self.table_name = table_name
        tbl = Table(table_name,
                    meta,
                    Column('id', VARCHAR(256), nullable=False,
                           primary_key=True),
                    Column('content', LargeBinary, nullable=True),
                    Column('d_expires', DateTime(timezone=True), nullable=True),
                    mysql_engine='InnoDB',
                    mysql_charset='utf8mb4')
        meta.create_all(db.connect(), tables=[tbl])

    def get(self, key, delete=False):
        """
        Get object from key-value storage

        Args:
            key: object key
            delete: delete object after getting
        Raises:
            LookupError: object not found
        """
        from msgpack import loads
        result = self.db.query('kv.get', qargs=[self.table_name],
                               id=key).fetchone()
        if result:
            if delete:
                self.delete(key)
            return loads(result.content, raw=False)
        else:
            raise LookupError

    def put(self, key=None, value=None, expires=None, override=True):
        """
        Put object to key-value storage

        If no key specified, random 64-char key is generated

        Args:
            key: string key (1-255 chars)
            value: value to put
            expires: expiration either in seconds or datetime.timedelta
            override: replace existing object
        Returns:
            object key
        """
        from msgpack import dumps
        if key is None:
            key = gen_random_str(length=64)
        elif override:
            try:
                self.delete(key)
            except LookupError:
                pass
        value = dumps(value)
        if expires is None:
            self.db.query('kv.put',
                          qargs=[self.table_name],
                          id=key,
                          content=value)
        else:
            d_expires = datetime.datetime.now() + datetime.timedelta(
                seconds=expires) if isinstance(
                    expires, int) else datetime.datetime.now() + expires
            self.db.query('kv.put.expires',
                          qargs=[self.table_name],
                          id=key,
                          content=value,
                          d_expires=d_expires)
        return key

    def delete(self, key):
        """
        Delete object in key-value storage

        Args:
            key: object key
        Raises:
            LookupError: object not found
        """
        if not self.db.query('kv.delete', qargs=[self.table_name],
                             id=key).rowcount:
            raise LookupError

    def cleanup(self):
        """
        Deletes expired objects
        """
        self.db.query('kv.cleanup',
                      qargs=[self.table_name],
                      d=datetime.datetime.now())



"""
Requires neotermcolor

neotermcolor styles:

- logger:10 - debug log message
- logger:20 - info log message
- logger:30 - warning log message
- logger:40 - error log message
- logger:50 - critical log message
- logger:exception - exceptions, printed to stdout (w/o logging)

Keeping records in memory requires neotasker library
"""
import logging
import logging.handlers
import platform
import neotermcolor
import threading
import time
import datetime
import types

from .network import parse_host_port

from types import SimpleNamespace

try:
    import rapidjson as json
except:
    import json

DEFAULT_LOG_GET = 100
MAX_LOG_GET = 10000

CLEAN_INTERVAL = 60

_exceptions = []

_log_records = []

_exception_log_lock = threading.RLock()
_log_record_lock = threading.RLock()

logger = logging.getLogger('pyaltt2.logs')

try:
    import pytz
    LOCAL_TZ = pytz.timezone(time.tzname[0])
except:
    logger.warning(
        'Unable to determine local time zone, is pytz module installed?')
    LOCAL_TZ = None


config = SimpleNamespace(
        name='',
        host=platform.node(),
        log_file=None,
        log_stdout=2,
        syslog=None,
        level=20,
        tracebacks=False,
        ignore=None,
        ignore_mods = [],
        stdout_ignore=True,
        keep_logmem=0,
        keep_exceptions=0,
        colorize=True,
        formatter = logging.Formatter('%(asctime)s ' + platform.node() + \
            ' %(levelname)s f:%(filename)s mod:%(module)s fn:%(funcName)s ' + \
            'l:%(lineno)d th:%(threadName)s :: %(message)s'),
        syslog_formatter = None,
        log_json=False,
        syslog_json=False
        )

__data = SimpleNamespace(logger=None, cleaner=None)

neotermcolor.set_style('logger:10', color='grey', attrs='bold')
neotermcolor.set_style('logger:20')
neotermcolor.set_style('logger:30', color='yellow')
neotermcolor.set_style('logger:40', color='red')
neotermcolor.set_style('logger:50', color='red', attrs='bold')

neotermcolor.set_style('logger:exception', color='red')


def _getJSONMessage(self):
    msg = str(self.msg)
    if self.args:
        msg = msg % self.args
    return json.dumps(msg)[1:-1]


class JSysLogHandler(logging.handlers.SysLogHandler):

    def __init__(self, *args, as_json=False, **kwargs):
        self.as_json = as_json
        super().__init__(*args, **kwargs)

    def emit(self, record):
        if self.as_json:
            record.getMessage = types.MethodType(_getJSONMessage, record)
        super().emit(record)


class JWatchedFileHandler(logging.handlers.WatchedFileHandler):

    def __init__(self, *args, as_json=False, **kwargs):
        self.as_json = as_json
        super().__init__(*args, **kwargs)

    def emit(self, record):
        if self.as_json:
            record.getMessage = types.MethodType(_getJSONMessage, record)
        super().emit(record)


class StdoutHandler(logging.StreamHandler):

    def __init__(self, as_json=False):
        self.as_json = as_json
        super().__init__()

    def emit(self, record):
        if not config.stdout_ignore or \
                ((config.ignore is None or \
                    not record.getMessage().startswith(config.ignore)) and \
                    record.module not in config.ignore_mods):
            if self.as_json:
                record.getMessage = types.MethodType(_getJSONMessage, record)
            super().emit(record)

    def format(self, record):
        r = super().format(record)
        return neotermcolor.colored(
            r, style='logger:' + str(record.levelno)) if config.colorize else r


def append(record=None, rd=None, **kwargs):
    """
    Append log record to memory cache

    Args:
        record: log record, or
        rd: log record in dict format
        **kwargs: passed to handle_append as-is
    """
    if record:
        r = {
            't': record.created,
            'msg': record.getMessage(),
            'l': record.levelno,
            'th': record.threadName,
            'mod': record.module,
            'h': config.host,
            'p': config.name
        }
    elif rd:
        r = rd
    else:
        return
    if r['msg'] and (not config.ignore or r['msg'][0] != config.ignore) and \
            r['mod'] not in config.ignore_mods:
        if LOCAL_TZ:
            r['dt'] = datetime.datetime.fromtimestamp(
                r['t']).replace(tzinfo=LOCAL_TZ).isoformat()
        with _log_record_lock:
            _log_records.append(r)
        handle_append(r, **kwargs)


def handle_append(rd, **kwargs):
    """
    Called after record is appended

    Args:
        rd: log record in dict format
        **kwargs: got from append as-is
    """


def get(level=0, t=0, n=None):
    """
    Get recent log records

    Args:
        level: minimal log level
        t: get entries for the recent t seconds
        n: max number of log records (default: 100)
    """
    lr = []
    if n is None:
        n = DEFAULT_LOG_GET
    if n > MAX_LOG_GET:
        n = MAX_LOG_GET
    t = time.time() - t if t else 0
    ll = 0 if level is None else level
    with _log_record_lock:
        recs = reversed(_log_records)
    for r in recs:
        if r['t'] > t and r['l'] >= ll:
            lr.append(r)
            if len(lr) >= n:
                break
    return list(reversed(lr))


async def clean(**kwargs):
    """
    Clean obsolete log records from memory

    Usually executed from log cleaner worker (see "start")
    """
    logger.debug('Cleaning logs')
    with _log_record_lock:
        recs = _log_records.copy()
    for l in recs:
        if time.time() - l['t'] > config.keep_logmem:
            with _log_record_lock:
                try:
                    _log_records.remove(l)
                except:
                    log_traceback()


class MemoryLogHandler(logging.Handler):

    def emit(self, record):
        append(record)


class DummyHandler(logging.StreamHandler):

    def emit(self, record):
        pass


def log_traceback(display=False, use_ignore=False, force=False, e=None):
    """
    Log exception traceback

    Args:
        display: display traceback instead of logging
        use_ignore: use ignore symbol for traceback string
        force: force log, even if tracebacks are disabled
        e: exception or exc_info to log (optional)
    """
    import traceback
    if e is None:
        e_msg = traceback.format_exc()
    elif isinstance(e, tuple):
        e_msg = ''.join(traceback.format_exception(*e))
    else:
        e_msg = str(e)
    if (config.tracebacks or force) and not display:
        pfx = config.ignore if use_ignore and config.ignore else ''
        logging.error(pfx + e_msg)
    elif display:
        print(neotermcolor.colored(e_msg, style='logger:exception'))
    if config.keep_exceptions:
        with _exception_log_lock:
            e = {
                't': datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S,%f'),
                'e': e_msg
            }
            _exceptions.append(e)
            if len(_exceptions) > config.keep_exceptions:
                del _exceptions[0]


def set_debug(debug=False):
    """
    Set debug mode ON/OFF

    Args:
        debug: True = ON, False = OFF
    """
    level = 10 if debug else config.level
    logging.basicConfig(level=level)
    if __data.logger:
        __data.logger.setLevel(level)


def serialize():
    """
    Get dict with internal data
    """
    with _exception_log_lock:
        return {'exceptions': _exceptions.copy()}


def init(**kwargs):
    """
    Initialize logger

    Args:
        name: software product name
        host: custom host name
        log_file: file to log to
        log_stdout: 0 - do not log, 1 - log, 2 - log auto (if no more log hdlrs)
        syslog: True for /dev/log, socket path or host[:port]
        level: log level (default: 20)
        tracebacks: log tracebacks (default: False)
        ignore: use "ignore" symbol - memory hdlr ignores records starting with
        ignore_mods: list of modules to ignore
        stdout_ignore: use "ignore" symbol in stdout logger as well
        keep_logmem: keep log records in memory for the specified time (seconds)
        keep_exceptions: keep number of recent exceptions
        colorize: colorize stdout if possible
        formatter: log formatter
        syslog_formatter: if defined, use custom formatter for syslog
        log_json: true/false
        syslog_json: true/false
    """
    for k, v in kwargs.items():
        if not hasattr(config, k):
            raise AttributeError('Invalid argument: {}'.format(k))
        setattr(config, k, v)

    logging.basicConfig(level=config.level)

    __data.logger = logging.getLogger()
    for h in __data.logger.handlers:
        __data.logger.removeHandler(h)
    has_handler = False
    if config.log_file:
        has_handler = True
        handler = JWatchedFileHandler(config.log_file, as_json=config.log_json)
        handler.setFormatter(config.formatter)
        __data.logger.addHandler(handler)
    if config.keep_logmem:
        handler = MemoryLogHandler()
        __data.logger.addHandler(handler)
    if config.syslog:
        has_handler = True
        if config.syslog is True:
            syslog_addr = '/dev/log'
        elif config.syslog.startswith('/'):
            syslog_addr = config.syslog
        else:
            addr, port = parse_host_port(config.syslog, 514)
            if addr:
                syslog_addr = (addr, port)
            else:
                logging.error('Invalid syslog configuration: {}'.format(
                    config.syslog))
                syslog_addr = None
        if syslog_addr:
            handler = JSysLogHandler(address=syslog_addr,
                                     as_json=config.syslog_json)
            handler.setFormatter(config.syslog_formatter if config.
                                 syslog_formatter else config.formatter)
            __data.logger.addHandler(handler)
    if (not has_handler and config.log_stdout == 2) or \
            config.log_stdout is True or config.log_stdout == 1:
        has_handler = True
        handler = StdoutHandler(as_json=config.log_json)
        handler.setFormatter(config.formatter)
        __data.logger.addHandler(handler)
    if not has_handler:
        # mute all logs
        __data.logger.addHandler(DummyHandler())


def start(loop=None):
    """
    Start log cleaner

    Requires neotasker module, task supervisor must be started before

    Args:
        loop: neotasker async loop to execute cleaner worker in
    """
    import neotasker
    __data.cleaner = neotasker.BackgroundIntervalWorker(
        name='pyaltt2:logs:cleaner', delay=CLEAN_INTERVAL, loop=loop)
    __data.cleaner.run = clean
    __data.cleaner.start()


def stop():
    """
    Optional method to stop log cleaner
    """
    if __data.cleaner:
        __data.cleaner.stop()
